<?php

namespace Drupal\ebt_carousel\Commands;

use Drush\Commands\DrushCommands;
use Symfony\Component\Filesystem\Filesystem;
use GuzzleHttp\Client;
use GuzzleHttp\Exception\RequestException;

/**
 * A Drush commandfile.
 *
 * In addition to this file, you need a drush.services.yml
 * in root of your module, and a composer.json file that provides the name
 * of the services file to use.
 *
 * See these files for an example of injecting Drupal services:
 *   - http://git.drupalcode.org/devel/tree/src/Commands/DevelCommands.php
 *   - http://git.drupalcode.org/devel/tree/drush.services.yml
 */
class TinySliderCommands extends DrushCommands {

  /**
   * Download and install the FlexSlider plugin.
   *
   * @param mixed $path
   *   Optional. A path where to install the FlexSlider plugin.
   *   If omitted Drush will use the default location.
   *
   * @command tinyslider:plugin
   * @aliases tinysliderplugin,tinyslider-plugin
   */
  public function download($path = '') {

    $fs = new Filesystem();

    if (empty($path)) {
      $path = DRUPAL_ROOT . '/libraries/tiny-slider';
    }

    // Create path if it doesn't exist
    // Exit with a message otherwise.
    if (!$fs->exists($path)) {
      $fs->mkdir($path);
    }
    else {
      $this->logger()->notice(dt('Tiny Slider is already present at @path. No download required.', ['@path' => $path]));
      return;
    }

    // Load the Tiny Slider defined library.
    if ($tinyslider_library = \Drupal::service('library.discovery')->getLibraryByName('ebt_carousel', 'tiny_slider')) {
      // Download the file.
      $client = new Client();
      $destination = tempnam(sys_get_temp_dir(), 'tiny-slider-tmp');
      try {
        $client->get($tinyslider_library['remote'] . '/archive/master.zip', ['save_to' => $destination]);
      }
      catch (RequestException $e) {
        // Remove the directory.
        $fs->remove($path);
        $this->logger()->error(dt('Drush was unable to download the Tiny Slider library from @remote. @exception', [
          '@remote' => $tinyslider_library['remote'] . '/archive/master.zip',
          '@exception' => $e->getMessage(),
        ], 'error'));
        return;
      }

      // Move downloaded file.
      $fs->rename($destination, $path . '/tiny-slider.zip');

      // Unzip the file.
      $zip = new \ZipArchive();
      $res = $zip->open($path . '/tiny-slider.zip');
      if ($res === TRUE) {
        $zip->extractTo($path);
        $zip->close();
      }
      else {
        // Remove the directory if unzip fails and exit.
        $fs->remove($path);
        $this->logger()->error(dt('Error: unable to unzip tiny slider file.', [], 'error'));
        return;
      }

      // Remove the downloaded zip file.
      $fs->remove($path . '/tiny-slider.zip');

      // Move the file.
      $fs->mirror($path . '/tiny-slider-master', $path, NULL, ['override' => TRUE]);
      $fs->remove($path . '/tiny-slider-master');

      // Success.
      $this->logger()->notice(dt('The Tiny Slider library has been successfully downloaded to @path.', [
        '@path' => $path,
      ], 'success'));
    }
    else {
      $this->logger()->error(dt('Drush was unable to load the Tiny Slider library'));
    }
  }

}
